#include <cmath>
#include <cstdlib>

#include "QuadAreaLight.hxx"

QuadAreaLight::QuadAreaLight(Scene* scene,
			     const Vec3f& intensity,
			     const Vec3f& p1,
			     const Vec3f& p2,
			     const Vec3f& p3,
			     const Vec3f& p4)
  : Light(scene),
    m_p0(p1),
    m_e1(p2-p1),
    m_e2(p3-p1),
    m_intensity(intensity)
{
  m_n = m_e1.cross(m_e2);
  m_area = m_n.norm();
  m_n.normalize();
}

QuadAreaLight::QuadAreaLight()
  : Light(0),
    m_p0(Vec3f()),
    m_e1(Vec3f()),
    m_e2(Vec3f()),
    m_n(Vec3f()),
    m_intensity(Vec3f())
{
}

QuadAreaLight::~QuadAreaLight()
{
}

bool
QuadAreaLight::Illuminate(Ray& shadow_ray, Vec3f& intensity)
{
  // generate random factors
  float xi1 = drand48();
  float xi2 = drand48();

  // bilinear interpolation of position
  Vec3f pos = m_p0 + m_e1 * xi1 + m_e2* xi2;
  // direction of shadow ray
  Vec3f dir = pos-shadow_ray.origin();

  float dist = dir.norm();
  float cosN = (dir.dot(m_n)) / dist;

  if(cosN <= 0.0f )
    return false;

  // squared falloff and projected area
  intensity = m_intensity * ( m_area * cosN / ( dist*dist));

  dir.normalize();
  shadow_ray.setDir(dir);
  shadow_ray.setT(dist);

  return true;
}

bool
QuadAreaLight::IsArea()
{
  return true;
}

const Vec3f&
QuadAreaLight::GetNormal(const Vec3f& pos) const
{
  return m_n;
}
